{% extends "base.html" %}

{% block content %}
<h1>wsgi_lineprof</h1>
<table class="table table-hover table-sm">
  <thead>
    <tr>
      <th scope="col">Started at</th>
      <th scope="col">Method</th>
      <th scope="col">Path</th>
      <th scope="col">Elapsed [ms]</th>
      <th scope="col">Actions</th>
    </tr>
  </thead>
  <tbody>
  {% for result in results %}
  <tr>
    <td>{{ result.started_at.strftime("%Y-%m-%dT%H:%M:%S%z") }}</td>
    <td>{{ result.request_method }}</td>
    <td>{{ result.path_info }}</td>
    <td class="text-right">{{ "%.03f"|format((1000 * result.elapsed)|round(3)) }}</td>
    <td>
      <a href="{{ result.id }}" role="button" class="btn btn-sm btn-primary">Show Result</a>
    </td>
  </tr>
  {% endfor %}
  </tbody>
</table>
{% endblock %}
